@@ -1,5 +1,6 @@
 /* Definitions for functions exported by the lexical analyser
-   Copyright (C) 1998,1999,2000,2001 James Bowman, Craig Franklin
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   James Bowman, Craig Franklin
 
 This file is part of gputils.
 
@@ -23,5 +24,7 @@ Boston, MA 02111-1307, USA.  */
 
 void open_src(char *name, int is_include_file);
 void close_file();
+void execute_exitm();
+void execute_macro(struct macro_head *h, int is_while);
 
 #endif
