@@ -1,5 +1,6 @@
 /* Linker script processing
-   Copyright (C) 2001 Craig Franklin
+   Copyright (C) 2001, 2002, 2003, 2004
+   Craig Franklin
  
 This file is part of gputils.
  
@@ -43,7 +44,8 @@ struct pnode {
   } value;
 };
 
-void gplkrscr_error(char *messg, char *detail);
+void script_error(char *messg, char *detail);
+int add_path(struct pnode *parms);
 int execute_command(char *name, struct pnode *parms);
 
 #endif
