@@ -1,5 +1,6 @@
 /* standard defines and includes
-   Copyright (C) 1998,1999,2000,2001 James Bowman, Craig Franklin
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004
+   James Bowman, Craig Franklin
    
 This file is part of gputils.
  
@@ -31,6 +32,11 @@ Boston, MA 02111-1307, USA.  */
 
 #include <stdlib.h>
 
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#define EXIT_FAILURE 1
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -57,6 +63,15 @@ Boston, MA 02111-1307, USA.  */
 #include <ctype.h>
 #include <errno.h>
 
+#ifdef HAVE_DOS_BASED_FILE_SYSTEM
+  #define PATH_CHAR '\\'
+  #define COPY_CHAR "\\"
+#else
+  #define PATH_CHAR '/'
+  #define COPY_CHAR "/"
+#endif
+
 #define BUG_REPORT_URL "<URL:http://gputils.sourceforge.net/>"
+#define GPUTILS_COPYRIGHT_STRING "Copyright (c) 1998-2004 gputils project"
 
 #endif
