@@ -1,5 +1,6 @@
 /* GNU PIC coff linker functions
-   Copyright (C) 2001 Craig Franklin
+   Copyright (C) 2001, 2002, 2003, 2004
+   Craig Franklin
 
 This file is part of gputils.
 
@@ -21,18 +22,6 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __GPCOFFLINK_H__
 #define __GPCOFFLINK_H__
 
-struct objectlist {
-  char              *name;
-  struct objectfile *object;
-  struct objectlist *next;
-};
-
-struct archivelist {
-  char                *name;
-  struct coff_archive *archive;
-  struct archivelist  *next;
-};
-
 /*
 
 accessbank - used for access registers in internal ram for 18CXX
@@ -59,27 +48,28 @@ struct linker_section {
   int fill;
   int use_fill;
   int protected;
-  /* working data */
-  int next_address;
 };
 
-struct coffsymbol {
-  struct syment     *coffsym;  /* the coff symbol */
-  struct objectfile *file;     /* the object file the symbol is defined in */
-  char              *filename; /* the name of the object file */
-};
+void gp_link_add_symbol(struct symbol_table *table,
+		        gp_symbol_type *symbol,
+		        gp_object_type *file);
 
-int gp_link_add_symbols(struct symbol_table *definition,
+void gp_link_remove_symbol(struct symbol_table *table, char *name);
+
+int gp_link_add_symbols(struct symbol_table *,
                         struct symbol_table *missing,
-                        struct objectfile *object);
+                        gp_object_type *object);
 
-void gp_link_reloc(struct objectlist *list,
-                   struct symbol_table *sections,
-                   struct symbol_table *logical_sections);
+void gp_cofflink_combine_overlay(gp_object_type *object, int remove_symbol);
 
-void gp_link_patch(struct objectlist *list,
-                   struct symbol_table *symbols);
+void gp_cofflink_reloc(gp_object_type *object,
+                       struct symbol_table *sections,
+                       struct symbol_table *logical_sections);
 
-struct objectfile *gp_link_combine(struct objectlist *list);
+void gp_cofflink_patch(gp_object_type *object, struct symbol_table *symbols);
+
+void gp_cofflink_clean_table(gp_object_type *object);
+
+MemBlock *gp_cofflink_make_memory(gp_object_type *object);
 
 #endif
